@@ -1,31 +1,43 @@
+import sys
 from typing import (
     Any,
     Dict,
     Iterator,
     List,
     Optional,
-    Protocol,
-    runtime_checkable,
     Sequence,
     Tuple,
     Union,
 )
+if sys.version_info.minor < 8:
+    from typing_extensions import Protocol, runtime_checkable
+else:
+    from typing import Protocol, runtime_checkable  # type: ignore
 
 from pydicom.dataset import Dataset
 
 
 @runtime_checkable
 class DICOMClient(Protocol):
-
     """Protocol for DICOM clients based on DICOMweb interface."""
 
+    base_url: str
+    scheme: str
+    protocol: Optional[str]
+    url_prefix: str
+    qido_url_prefix: Optional[str] = None
+    wado_url_prefix: Optional[str] = None
+    stow_url_prefix: Optional[str] = None
+    delete_url_prefix: Optional[str] = None
+
     def search_for_studies(
         self,
         fuzzymatching: Optional[bool] = None,
         limit: Optional[int] = None,
         offset: Optional[int] = None,
         fields: Optional[Sequence[str]] = None,
-        search_filters: Optional[Dict[str, Any]] = None
+        search_filters: Optional[Dict[str, Any]] = None,
+        get_remaining: bool = False
     ) -> List[Dict[str, dict]]:
         """Search for studies.
 
@@ -43,6 +55,9 @@ class DICOMClient(Protocol):
             Search filter criteria as key-value pairs, where *key* is a keyword
             or a tag of the attribute and *value* is the expected value that
             should match
+        get_remaining: bool, optional
+            Whether remaining results should be included (this may repeatedly
+            query the server for remaining results)
 
         Returns
         -------
@@ -57,13 +72,13 @@ class DICOMClient(Protocol):
         Remaining results can be requested via repeated calls using the
         `offset` parameter.
 
-        """  # noqa: E501: E501
+        """  # noqa: E501
         pass
 
     def retrieve_bulkdata(
         self,
         url: str,
-        media_types: Optional[Tuple[Union[str, Tuple[str, str]]]] = None,
+        media_types: Optional[Tuple[Union[str, Tuple[str, str]], ...]] = None,
         byte_range: Optional[Tuple[int, int]] = None
     ) -> List[bytes]:
         """Retrieve bulk data at a given location.
@@ -72,7 +87,7 @@ class DICOMClient(Protocol):
         ----------
         url: str
             Location of the bulk data
-        media_types: Union[Tuple[Union[str, Tuple[str, str]]], None], optional
+        media_types: Union[Tuple[Union[str, Tuple[str, str]], ...], None], optional
             Acceptable media types and optionally the UIDs of the
             corresponding transfer syntaxes
         byte_range: Union[Tuple[int, int], None], optional
@@ -83,13 +98,13 @@ class DICOMClient(Protocol):
         List[bytes]
             Bulk data items
 
-        """
+        """  # noqa: E501
         pass
 
     def iter_bulkdata(
         self,
         url: str,
-        media_types: Optional[Tuple[Union[str, Tuple[str, str]]]] = None,
+        media_types: Optional[Tuple[Union[str, Tuple[str, str]], ...]] = None,
         byte_range: Optional[Tuple[int, int]] = None
     ) -> Iterator[bytes]:
         """Iterate over bulk data items at a given location.
@@ -98,7 +113,7 @@ class DICOMClient(Protocol):
         ----------
         url: str
             Location of the bulk data
-        media_types: Union[Tuple[Union[str, Tuple[str, str]]], None], optional
+        media_types: Union[Tuple[Union[str, Tuple[str, str]], ...], None], optional
             Acceptable media types and optionally the UIDs of the
             corresponding transfer syntaxes
         byte_range: Union[Tuple[int, int], None], optional
@@ -109,13 +124,13 @@ class DICOMClient(Protocol):
         Iterator[bytes]
             Bulk data items
 
-        """
+        """  # noqa: E501
         pass
 
     def retrieve_study(
         self,
         study_instance_uid: str,
-        media_types: Optional[Tuple[Union[str, Tuple[str, str]]]] = None,
+        media_types: Optional[Tuple[Union[str, Tuple[str, str]], ...]] = None,
     ) -> List[Dataset]:
         """Retrieve all instances of a study.
 
@@ -123,7 +138,7 @@ class DICOMClient(Protocol):
         ----------
         study_instance_uid: str
             Study Instance UID
-        media_types: Union[Tuple[Union[str, Tuple[str, str]]], None], optional
+        media_types: Union[Tuple[Union[str, Tuple[str, str]], ...], None], optional
             Acceptable media types and optionally the UIDs of the
             acceptable transfer syntaxes
 
@@ -142,13 +157,13 @@ class DICOMClient(Protocol):
         acceptable transfer syntaxes using the wildcard
         ``("application/dicom", "*")``.
 
-        """
+        """  # noqa: E501
         pass
 
     def iter_study(
         self,
         study_instance_uid: str,
-        media_types: Optional[Tuple[Union[str, Tuple[str, str]]]] = None,
+        media_types: Optional[Tuple[Union[str, Tuple[str, str]], ...]] = None,
     ) -> Iterator[Dataset]:
         """Iterate over all instances of a study.
 
@@ -156,7 +171,7 @@ class DICOMClient(Protocol):
         ----------
         study_instance_uid: str
             Study Instance UID
-        media_types: Union[Tuple[Union[str, Tuple[str, str]]], None], optional
+        media_types: Union[Tuple[Union[str, Tuple[str, str]], ...], None], optional
             Acceptable media types and optionally the UIDs of the
             acceptable transfer syntaxes
 
@@ -175,7 +190,7 @@ class DICOMClient(Protocol):
         acceptable transfer syntaxes using the wildcard
         ``("application/dicom", "*")``.
 
-        """
+        """  # noqa: E501
         pass
 
     def retrieve_study_metadata(
@@ -225,7 +240,8 @@ class DICOMClient(Protocol):
         limit: Optional[int] = None,
         offset: Optional[int] = None,
         fields: Optional[Sequence[str]] = None,
-        search_filters: Optional[Dict[str, Any]] = None
+        search_filters: Optional[Dict[str, Any]] = None,
+        get_remaining: bool = False
     ) -> List[Dict[str, dict]]:
         """Search for series.
 
@@ -245,6 +261,9 @@ class DICOMClient(Protocol):
             Search filter criteria as key-value pairs, where *key* is a keyword
             or a tag of the attribute and *value* is the expected value that
             should match
+        get_remaining: bool, optional
+            Whether remaining results should be included (this may repeatedly
+            query the server for remaining results)
 
         Returns
         -------
@@ -266,7 +285,7 @@ class DICOMClient(Protocol):
         self,
         study_instance_uid: str,
         series_instance_uid: str,
-        media_types: Optional[Tuple[Union[str, Tuple[str, str]]]] = None
+        media_types: Optional[Tuple[Union[str, Tuple[str, str]], ...]] = None
     ) -> List[Dataset]:
         """Retrieve all instances of a series.
 
@@ -276,7 +295,7 @@ class DICOMClient(Protocol):
             Study Instance UID
         series_instance_uid: str
             Series Instance UID
-        media_types: Union[Tuple[Union[str, Tuple[str, str]]], None], optional
+        media_types: Union[Tuple[Union[str, Tuple[str, str]], ...], None], optional
             Acceptable media types and optionally the UIDs of the
             acceptable transfer syntaxes
 
@@ -295,14 +314,14 @@ class DICOMClient(Protocol):
         acceptable transfer syntaxes using the wildcard
         ``("application/dicom", "*")``.
 
-        """
+        """  # noqa: E501
         pass
 
     def iter_series(
         self,
         study_instance_uid: str,
         series_instance_uid: str,
-        media_types: Optional[Tuple[Union[str, Tuple[str, str]]]] = None
+        media_types: Optional[Tuple[Union[str, Tuple[str, str]], ...]] = None
     ) -> Iterator[Dataset]:
         """Iterate over all instances of a series.
 
@@ -312,7 +331,7 @@ class DICOMClient(Protocol):
             Study Instance UID
         series_instance_uid: str
             Series Instance UID
-        media_types: Union[Tuple[Union[str, Tuple[str, str]]], None], optional
+        media_types: Union[Tuple[Union[str, Tuple[str, str]], ...], None], optional
             Acceptable media types and optionally the UIDs of the
             acceptable transfer syntaxes
 
@@ -331,7 +350,7 @@ class DICOMClient(Protocol):
         acceptable transfer syntaxes using the wildcard
         ``("application/dicom", "*")``.
 
-        """
+        """  # noqa: E501
         pass
 
     def retrieve_series_metadata(
@@ -359,7 +378,7 @@ class DICOMClient(Protocol):
     def retrieve_series_rendered(
         self, study_instance_uid,
         series_instance_uid,
-        media_types: Optional[Tuple[Union[str, Tuple[str, str]]]] = None,
+        media_types: Optional[Tuple[Union[str, Tuple[str, str]], ...]] = None,
         params: Optional[Dict[str, Any]] = None
     ) -> bytes:
         """Retrieve rendered representation of a series.
@@ -370,7 +389,7 @@ class DICOMClient(Protocol):
             Study Instance UID
         series_instance_uid: str
             Series Instance UID
-        media_types: Union[Tuple[Union[str, Tuple[str, str]]], None], optional
+        media_types: Union[Tuple[Union[str, Tuple[str, str]], ...], None], optional
             Acceptable media types (choices: ``"image/jpeg"``, ``"image/jp2"``,
             ``"image/gif"``, ``"image/png"``, ``"video/gif"``, ``"video/mp4"``,
             ``"video/h265"``, ``"text/html"``, ``"text/plain"``,
@@ -384,7 +403,7 @@ class DICOMClient(Protocol):
         bytes
             Rendered representation of series
 
-        """
+        """  # noqa: E501
         pass
 
     def delete_series(
@@ -422,7 +441,8 @@ class DICOMClient(Protocol):
         limit: Optional[int] = None,
         offset: Optional[int] = None,
         fields: Optional[Sequence[str]] = None,
-        search_filters: Optional[Dict[str, Any]] = None
+        search_filters: Optional[Dict[str, Any]] = None,
+        get_remaining: bool = False
     ) -> List[Dict[str, dict]]:
         """Search for instances.
 
@@ -444,6 +464,9 @@ class DICOMClient(Protocol):
             Search filter criteria as key-value pairs, where *key* is a keyword
             or a tag of the attribute and *value* is the expected value that
             should match
+        get_remaining: bool, optional
+            Whether remaining results should be included (this may repeatedly
+            query the server for remaining results)
 
         Returns
         -------
@@ -466,7 +489,7 @@ class DICOMClient(Protocol):
         study_instance_uid: str,
         series_instance_uid: str,
         sop_instance_uid: str,
-        media_types: Optional[Tuple[Union[str, Tuple[str, str]]]] = None,
+        media_types: Optional[Tuple[Union[str, Tuple[str, str]], ...]] = None,
     ) -> Dataset:
         """Retrieve an individual instance.
 
@@ -478,7 +501,7 @@ class DICOMClient(Protocol):
             Series Instance UID
         sop_instance_uid: str
             SOP Instance UID
-        media_types: Union[Tuple[Union[str, Tuple[str, str]]], None], optional
+        media_types: Union[Tuple[Union[str, Tuple[str, str]], ...], None], optional
             Acceptable media types and optionally the UIDs of the
             acceptable transfer syntaxes
 
@@ -497,7 +520,7 @@ class DICOMClient(Protocol):
         acceptable transfer syntaxes using the wildcard
         ``("application/dicom", "*")``.
 
-        """
+        """  # noqa: E501
         pass
 
     def store_instances(
@@ -582,7 +605,7 @@ class DICOMClient(Protocol):
         study_instance_uid: str,
         series_instance_uid: str,
         sop_instance_uid: str,
-        media_types: Optional[Tuple[Union[str, Tuple[str, str]]]] = None,
+        media_types: Optional[Tuple[Union[str, Tuple[str, str]], ...]] = None,
         params: Optional[Dict[str, Any]] = None
     ) -> bytes:
         """Retrieve an individual, server-side rendered instance.
@@ -595,7 +618,7 @@ class DICOMClient(Protocol):
             Series Instance UID
         sop_instance_uid: str
             SOP Instance UID
-        media_types: Union[Tuple[Union[str, Tuple[str, str]]], None], optional
+        media_types: Union[Tuple[Union[str, Tuple[str, str]], ...], None], optional
             Acceptable media types (choices: ``"image/jpeg"``, ``"image/jp2"``,
             ``"image/gif"``, ``"image/png"``, ``"video/gif"``, ``"video/mp4"``,
             ``"video/h265"``, ``"text/html"``, ``"text/plain"``,
@@ -609,7 +632,7 @@ class DICOMClient(Protocol):
         bytes
             Rendered representation of instance
 
-        """
+        """  # noqa: E501
         pass
 
     def retrieve_instance_frames(
@@ -618,7 +641,7 @@ class DICOMClient(Protocol):
         series_instance_uid: str,
         sop_instance_uid: str,
         frame_numbers: Sequence[int],
-        media_types: Optional[Tuple[Union[str, Tuple[str, str]]]] = None
+        media_types: Optional[Tuple[Union[str, Tuple[str, str]], ...]] = None
     ) -> List[bytes]:
         """Retrieve one or more frames of an image instance.
 
@@ -632,7 +655,7 @@ class DICOMClient(Protocol):
             SOP Instance UID
         frame_numbers: Sequence[int]
             One-based positional indices of the frames within the instance
-        media_types: Union[Tuple[Union[str, Tuple[str, str]]], None], optional
+        media_types: Union[Tuple[Union[str, Tuple[str, str]], ...], None], optional
             Acceptable media types and optionally the UIDs of the
             corresponding transfer syntaxes
 
@@ -641,7 +664,7 @@ class DICOMClient(Protocol):
         List[bytes]
             Pixel data for each frame
 
-        """
+        """  # noqa: E501
         pass
 
     def iter_instance_frames(
@@ -650,7 +673,7 @@ class DICOMClient(Protocol):
         series_instance_uid: str,
         sop_instance_uid: str,
         frame_numbers: Sequence[int],
-        media_types: Optional[Tuple[Union[str, Tuple[str, str]]]] = None
+        media_types: Optional[Tuple[Union[str, Tuple[str, str]], ...]] = None
     ) -> Iterator[bytes]:
         """Iterate over frames of an image instance.
 
@@ -664,7 +687,7 @@ class DICOMClient(Protocol):
             SOP Instance UID
         frame_numbers: Sequence[int]
             One-based positional indices of the frames within the instance
-        media_types: Union[Tuple[Union[str, Tuple[str, str]]], None], optional
+        media_types: Union[Tuple[Union[str, Tuple[str, str]], ...], None], optional
             Acceptable media types and optionally the UIDs of the
             corresponding transfer syntaxes
 
@@ -673,7 +696,7 @@ class DICOMClient(Protocol):
         Iterator[bytes]
             Pixel data for each frame
 
-        """
+        """  # noqa: E501
         pass
 
     def retrieve_instance_frames_rendered(
@@ -682,7 +705,7 @@ class DICOMClient(Protocol):
         series_instance_uid: str,
         sop_instance_uid: str,
         frame_numbers: Sequence[int],
-        media_types: Optional[Tuple[Union[str, Tuple[str, str]]]] = None,
+        media_types: Optional[Tuple[Union[str, Tuple[str, str]], ...]] = None,
         params: Optional[Dict[str, Any]] = None
     ) -> bytes:
         """Retrieve one or more server-side rendered frames of an instance.
@@ -697,7 +720,7 @@ class DICOMClient(Protocol):
             SOP Instance UID
         frame_numbers: Sequence[int]
             One-based positional index of the frame within the instance
-        media_types: Union[Tuple[Union[str, Tuple[str, str]]], None], optional
+        media_types: Union[Tuple[Union[str, Tuple[str, str]], ...], None], optional
             Acceptable media type (choices: ``"image/jpeg"``, ``"image/jp2"``,
             ``"image/gif"``, ``"image/png"``)
         params: Union[Dict[str, Any], None], optional
@@ -713,5 +736,5 @@ class DICOMClient(Protocol):
         ----
         Not all media types are compatible with all SOP classes.
 
-        """
+        """  # noqa: E501
         pass
